Package: scDD
Version: 1.34.0
Title: Mixture modeling of single-cell RNA-seq data to identify genes
        with differential distributions
Description: This package implements a method to analyze single-cell
        RNA- seq Data utilizing flexible Dirichlet Process mixture
        models. Genes with differential distributions of expression are
        classified into several interesting patterns of differences
        between two conditions. The package also includes functions for
        simulating data with these patterns from negative binomial
        distributions.
Authors@R: person("Keegan", "Korthauer", role = c("cre", "aut"), 
                  email = "keegan@stat.ubc.ca", 
                  comment = c(ORCID = "0000-0002-4565-1654"))
Depends: R (>= 3.5.0)
NeedsCompilation: yes
Imports: fields, mclust, BiocParallel, outliers, ggplot2, EBSeq, arm,
        SingleCellExperiment, SummarizedExperiment, grDevices,
        graphics, stats, S4Vectors, scran
Suggests: BiocStyle, knitr, gridExtra
License: GPL-2
RoxygenNote: 6.0.1
VignetteBuilder: knitr
biocViews: ImmunoOncology, Bayesian, Clustering, RNASeq, SingleCell,
        MultipleComparison, Visualization, DifferentialExpression
URL: https://github.com/kdkorthauer/scDD
BugReports: https://github.com/kdkorthauer/scDD/issues
git_url: https://git.bioconductor.org/packages/scDD
git_branch: RELEASE_3_22
git_last_commit: 924aec4
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
Packaged: 2025-10-30 06:18:00 UTC; biocbuild
Author: Keegan Korthauer [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-4565-1654>)
Maintainer: Keegan Korthauer <keegan@stat.ubc.ca>
